@@ -25,11 +25,12 @@ import CppIfdef (cppIfdef)
 -- | Get the imports for this Haskell module.
 getImports :: FilePath -- ^ The path to the module
            -> [String] -- ^ Definitions, from which to build a symbol table (for cpp)
+           -> [String] -- ^ Search-path for #include'd files
            -> String   -- ^ The input file to be parsed for imports
            -> [String] -- ^ A list of imported modules
-getImports fp defines = leximports fp
-                        . unlines . map snd
-                        . cppIfdef fp defines [] False False
+getImports fp defines includes = leximports fp
+                                 . unlines . map snd
+                                 . cppIfdef fp defines includes False False
 
 
 -- | /leximports/ takes a cpp-ed input and returns the list of imports
@@ -55,6 +56,7 @@ leximports fp =
     nestcomment 0 ('\'':'"':'\'':cs)  = '\'':'"':'\'': nestcomment 0 cs
     nestcomment 0 ('\\':'"':cs)       = '\\':'"': nestcomment 0 cs
     nestcomment 0 ('"':cs)            = '"': endstring cs
+    nestcomment 0 (';':cs)            = '\n': nestcomment 0 cs	-- HACK
     nestcomment 0 (c:cs)              = c: nestcomment 0 cs
     nestcomment 0 []                  = []
     nestcomment n []                  =
@@ -69,14 +71,24 @@ leximports fp =
     endstring []       = []
     stringgap ('\\':cs) = endstring cs
     stringgap (c:cs)    = stringgap cs
+    stringgap []        = []	-- source file is corrupt
 
     getmodnames (x:xs)
       | null x || all isSpace x  = getmodnames xs
       | otherwise =
-        let ws = concatMap words (x:xs)	-- allow for import spanning several lines.
-        in if not (null ws) && head ws == "import" then
-               modname (tail ws): getmodnames xs
-           else getmodnames xs
+        case concatMap words (x:xs) of
+          ("import":ws)  -> modname ws: getmodnames xs
+				-- allow for import spanning several lines.
+          ("data":ws)    -> []	-- truncate search at first non-import keyword
+          ("type":ws)    -> []
+          ("class":ws)   -> []
+          ("infix":ws)   -> []
+          ("infixl":ws)  -> []
+          ("infixr":ws)  -> []
+          ("newtype":ws) -> []
+          ("default":ws) -> []
+          ("instance":ws)-> []
+          _              -> getmodnames xs	-- non-keyword: continue search
     getmodnames [] = []
 
     modname ws =
