@@ -1,6 +1,6 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Main
+-- Module      :  MkConfig
 -- Copyright   :  Malcolm Wallace
 -- 
 -- Maintainer  :  Malcolm Wallace <Malcolm.Wallace@cs.york.ac.uk>
@@ -12,17 +12,12 @@
 
 module Main where
 
-import Compiler (HC(..))
 import Config
-import Platform (unsafePerformIO,escape,windows,exe)
-import RunAndReadStdout (runAndReadStdout, basename, dirname)
-import Directory (doesDirectoryExist,doesFileExist,removeFile,getPermissions
-                 ,Permissions(..),renameFile,createDirectory)
-import System (exitWith,ExitCode(..),getArgs,getEnv,getProgName)
-import List (intersperse,nub,isPrefixOf,sort)
-import Char (isDigit)
-import Monad (foldM,when)
-import Maybe (isJust,fromJust)
+import RunAndReadStdout (dirname)
+import Directory (createDirectory)
+import System (exitWith,ExitCode(..),getArgs)
+import List (nub,sort)
+import Maybe (fromJust)
 import IO (stderr,isDoesNotExistError)
 #ifdef __HBC__
 import IOMisc (hPutStrLn)
@@ -44,30 +39,31 @@ main = do
                    (case lfile of
                       Just f -> putStrLn ("Personal config file is:\n    "++f)
                       Nothing -> return ())
+                   known <- mapM unDyn $ knownComps config
                    putStrLn "Known compilers:"
                    mapM_ putStrLn
                          ((reverse . sort
                            . map (\c-> "    "++compilerPath c
                                        ++"\t("++compilerVersion c++")"))
-                          (knownComps config))
+                          known)
                    putStrLn "Default compiler:"
                    putStrLn ("    "++defaultComp config)
     [hc] -> do -- no command, assume 'add'
-               cc <- configure (hcStyle hc) hc
+               cc <- configure hc
                config' <- add cc config
                writeBack gfile lfile config'
-    ["add",hc]     -> do cc <- configure (hcStyle hc) hc
+    ["add",hc]     -> do cc <- configure hc
                          config' <- add cc config
                          writeBack gfile lfile config'
+    ["add-dyn",hc] -> do config' <- add (DynCompiler hc) config
+                         writeBack gfile lfile config'
     ["delete",hc]  -> do config' <- delete config gfile hc
                          writeBack gfile lfile config'
     ["default",hc] -> do config' <- mkDefault config hc
                          writeBack gfile lfile config'
     ["list",hc]    -> do let cc = matchCompiler hc config
                          putStrLn (show cc)
-    _ -> do hPutStrLn stderr ("Usage: hmake-config [configfile] list\n"
-                 ++"       hmake-config [configfile] [add|delete|default] hc\n"
-                 ++"                  -- hc is name/path of a Haskell compiler")
+    _ -> do hPutStrLn stderr usage
             exitWith (ExitFailure 1)
   ----
   exitWith ExitSuccess
@@ -77,10 +73,7 @@ main = do
     findConfigFile args =
       case args of
         [] -> do let (g,_) = defaultConfigLocation False
-                 hPutStrLn stderr ("Usage: hmake-config [configfile] list\n"
-                  ++"       hmake-config [configfile] [add|delete|default] hc\n"
-                  ++"              -- hc is name/path of a Haskell compiler\n"
-                  ++"  default configfile is:\n    "++g)
+                 hPutStrLn stderr (usage++"\ndefault configfile is:\n    "++g)
                  exitWith (ExitFailure 1)
         (file:"new":_)  -> return (file, Nothing, tail args)
         (file:"list":_) -> return (file, Nothing, tail args)
@@ -88,6 +81,9 @@ main = do
         ("list":_) ->
              let (g,l) = defaultConfigLocation False in return (g, l, args)
         _ -> let (g,l) = defaultConfigLocation True in return (g, l, args)
+    usage = "Usage: hmake-config [configfile] list\n"
+         ++ "       hmake-config [configfile] [add|add-dyn|delete|default] hc\n"
+         ++ "              -- hc is name/path of a Haskell compiler"
 
 {-
     parseConfigFile :: String -> FilePath -> IO HmakeConfig
@@ -98,7 +94,7 @@ main = do
                     hPutStrLn stderr ("hmake-config: Warning: "
                                       ++"Config file not found:\n  '"
                                       ++path++"'")
-                    globalDir <- getEnv "HMAKEDIR"
+                    globalDir <- getEnv "HMAKECONFDIR"
                     let global = globalDir++"/"++machine++"/hmakerc"
                     if path == global
                       then newConfigFile path
@@ -207,6 +203,7 @@ add hc config = return $
                           global { knownCompilers =
                                              nub (hc: knownCompilers global)}}
 
+{-
 -- | configure for each style of compiler
 configure :: HC -> String -> IO CompilerConfig
 configure Ghc ghcpath = do
@@ -250,9 +247,8 @@ configure Ghc ghcpath = do
             else do ioError (userError ("Can't find ghc includes at\n  "
                                         ++incdir1++"\n  "++incdir2))
     else do -- 5.00 and above
-      pkgcfg <- runAndReadStdout (escape ghcpath++" -v 2>&1 | head -2 "
-                                  ++"| tail -1 | cut -c28- | head -1")
-      let libdir  = dirname (escape pkgcfg)
+      pkgcfg <- runAndReadStdout (escape ghcpath++" --print-libdir")
+      let libdir  = escape pkgcfg
           incdir1 = libdir++"/imports"
       ok <- doesDirectoryExist incdir1
       if ok
@@ -263,9 +259,12 @@ configure Ghc ghcpath = do
           let ghcpkg = if ok then ghcpkg0 else dirname fullpath++"/ghc-pkg"
        -- pkgs <- runAndReadStdout (ghcpkg++" --list-packages")
           pkgs <- runAndReadStdout (ghcpkg++" -l")
-          let pkgsOK = filter (`elem`["std","base","haskell98"]) (deComma pkgs)
+          let pkgsOK = filter (\p-> any (`isPrefixOf` p)
+                                        ["std","base","haskell98"])
+                              (deComma pkgs)
           idirs <- mapM (\p-> runAndReadStdout
-                                  (ghcpkg++" --show-package="++p
+                                  (ghcpkg++" --show-package="
+                                   ++deVersion (ghcsym>=604) p
                                    ++" --field=import_dirs"))
                         pkgsOK
           return config{ includePaths = pkgDirs libdir (nub idirs) }
@@ -282,9 +281,14 @@ configure Ghc ghcpath = do
     pkgDirs libdir dirs =
         map (\dir-> if "$libdir" `isPrefixOf` dir
                     then libdir++drop 7 dir
+                    else if "[\"" `isPrefixOf` dir
+                    then drop 2 (init (init dir))
                     else dir)
             (concatMap words dirs)
     deComma pkgs = map (\p-> if last p==',' then init p else p) (words pkgs)
+    deVersion False pkg = pkg
+    deVersion True  pkg = let (suf,pref) = span (/='-') (reverse pkg)
+                          in case pref of "" -> pkg; _ -> reverse (tail pref)
 
 configure Nhc98 nhcpath = do
   fullpath <- which id nhcpath
@@ -372,4 +376,4 @@ which exe cmd =
     perms file = do
         p <- getPermissions file
         return (if executable p then Just file else Nothing)
-
+-}
